﻿/*
 * FLASHAIR.c
 *
 * Created: 2014/10/21 22:42:32
 *  Author: 浩一
 */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <string.h>
#include <avr/eeprom.h>
#include <util/delay.h>
#include "SD.h"
#include "BUFFER.h"
#include "MOUNT.h"
#include "LCD.h"
#include "COMMON.h"
#include "SD6031CORE.h"
#include "MESSAGE.h"

uint8_t isFlashAir = 0;

// examine whether the card is FlashAir or not. must be called right after the first SD initialization
uint8_t IS_FLASHAIR(void)
{
	uint8_t err = 0;
	struct FILE tempfile;

	char *config = (char *)buffer2.wifi.short_name;
	strcpy (config, "SD_WLAN    CONFIG     ");	
	// open the file with the absolute path
	// name should be <8 character directory name><3 character extension><8...><3...>...0
	FILE_openAbs(&tempfile, config, 1, &err);
	return !err;
}

void FLASHAIR_set_status(void)
{
	uint8_t err = 0;

	SD_writeSharedMemoryBegin(0, 512, &err);
	// 0x0054 = 'T' = status command
	SPI_writeByte('T', &err);
	SPI_writeByte(1, &err);
	SPI_writeByte(0, &err);
	SPI_writeByte(0, &err);
	SPI_writeByte(0, &err);
	SPI_writeByte(0, &err);
	SPI_writeByte((globalProtect[0]?0x10:0)|(globalProtect[1]?0x20:0)
			|(globalProtect[2]?0x40:0)|(globalProtect[3]?0x80:0), &err);
	SPI_writeByte(FIRMWRITTEN, &err);
	for (uint8_t drv=0; drv<4; drv++) {
		struct FILE *imgp = &buffer2.sd6031.img[drv];
		if (imgp->valid) {
			for (uint8_t i=0; i<8; i++) SPI_writeByte(imgp->name[i], &err);
		} else {
			for (uint8_t i=0; i<8; i++) SPI_writeByte(' ', &err);
		}
	}
#ifdef DEBUG
	for (uint16_t i=0; i<(64-40); i++) SPI_writeByte(0, &err);
	for (uint16_t i=0; i<64; i++) SPI_writeByte(message[i], &err);
	for (uint16_t i=0; i<(512-128); i++) SPI_writeByte(0, &err);
#else
	for (uint16_t i=0; i<(512-40); i++) SPI_writeByte(0, &err);
#endif
	SD_writeSharedMemoryEnd(&err);
}

// check and execute a command from shared memory
void FLASHAIR_command(void)
{
	uint8_t err = 0;

	cli();

	// read first word (command)
	SD_readSharedMemoryBegin(0, 512, &err);
	uint8_t cmd = SPI_readByte(&err);
	uint8_t param = SPI_readByte(&err)-'0';
	uint8_t param2 = SPI_readByte(&err)-'0';
	for (uint16_t i=0; i<509; i++) SPI_readByte(&err);
	SD_readSharedMemoryEnd(&err);
	
	switch (cmd) {
		// global write protect command
		case 'P': {
			globalProtect[param&0b11] = ((param&0b100)?1:0);
			break;
		}
		// drive change command
		case 'D': {
			FILE_getShortFileName(buffer2.wifi.short_name, &err);
			if (*(char *)buffer2.wifi.short_name) MOUNT_mount(param/*drive*/, (char *)buffer2.wifi.short_name);
			break;
		}
		// unmount command
		case 'U': {
			strcpy((char *)buffer2.wifi.short_name, "           ");
			MOUNT_mount(param/*drive*/, (char *)buffer2.wifi.short_name);
			break;	
		}
		// write back command
		case 'W': {
			// write back to d88 file
			MOUNT_writeBack(param/*drive*/);
			break;
		}		
	}
	FLASHAIR_set_status();
	sei();
}