﻿/*
 * MOUNT.c
 *
 * Created: 2013/10/10 21:26:42
 *  Author: Koichi Nishida
 */
/*
Copyright (C) 2016 Koichi NISHIDA
email to Koichi NISHIDA: tulip-house@msf.biglobe.ne.jp

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include "COMMON.h"
#include "MOUNT.h"
#include "SD.h"
#include "LCD.h"
#include "BUFFER.h"
#include "INI.h"
#include "D88_P31.h"
#include "FLASHAIR.h"
#include <string.h>

// write back to the D88 image
void MOUNT_writeBack(uint8_t drv)
{
	if (!WP) {
		uint8_t err = 0;
		INI_read(buffer2.ini.ini, &err);
		if (!err) {
			if (buffer2.sd6031.img[drv].valid) {
				FILE_openAbs(&buffer2.sd6031.d88[drv], (char *)buffer2.ini.ini+drv*64, 0, &err);
				if (!err && !buffer2.sd6031.d88[drv].protect && (memcmp(&buffer2.sd6031.d88[drv].name[8], "D88", 3)==0)) {
					p312d88(&buffer2.sd6031.d88[drv], &buffer2.sd6031.img[drv], &err);
				}
			}
		}
	}
}

// mount an image in the FlashAir to the drive
void MOUNT_mount(uint8_t drive, char *short_name)
{	
	uint8_t err = 0;
#ifdef AIRDISP
	LCD_cls();
	LCD_locate(0,1);
	LCD_print(short_name+(strlen(short_name)-11), 8);
#endif
	INI_substitute(buffer2.ui.filelist/*this is just buffer*/, drive, short_name, &err);
	if (memcmp(short_name, "           ",11) == 0) {
		buffer2.sd6031.img[drive].valid = 0; 
	} else {
		FILE_openAbs(&buffer2.sd6031.d88[drive], short_name, 0, &err);
		if (err) { buffer2.sd6031.img[drive].valid = 0; buffer2.sd6031.d88[drive].valid = 0; }
		else if (memcmp(&buffer2.sd6031.d88[drive].name[8], "P31", 3) == 0) {
			buffer2.sd6031.img[drive]=buffer2.sd6031.d88[drive];
			buffer2.sd6031.d88[drive].valid = 0;
		} else {
			FILE_substituteFullpathExtwith(short_name, "P31");
			FILE_openAbs(&buffer2.sd6031.img[drive], short_name, 0, &err);
			if (err) {
				if (!WP) {
					err = 0;
#ifdef AIRDISP
					LCD_printAll("CreatingP31 file");
#endif
					// Notice : buffer2.sd.buf[512] is also used!
					cli();
					FILE_createAbs(&buffer2.sd6031.img[drive], short_name, 655360, &err);
					sei();
					if (!err) {
						FILE_openAbs(&buffer2.sd6031.img[drive], short_name, 0, &err);
						buffer2.sd6031.img[drive].protect = buffer2.sd6031.d88[drive].protect;	
						d882p31(&buffer2.sd6031.img[drive], &buffer2.sd6031.d88[drive], &err);
						if (err) buffer2.sd6031.img[drive].valid = 0;
					}
				} else {
#ifdef AIRDISP
					LCD_printAll("Write   protect.");
#endif				
				}
			}
		}
	}
}

// operation for exchanging the SD card
void MOUNT_exchangeSD(uint8_t start)
{
	uint8_t err = 0;

	if (EJECT || start) {
		for (uint8_t drv = 0; drv < 4; drv++) {
			buffer2.sd6031.img[drv].valid = 0;
			buffer2.sd6031.d88[drv].valid = 0;
		}
		SD_p.inited = 0;
	}
	if (!SD_detect(start)) return;

	isFlashAir = IS_FLASHAIR();
	if (isFlashAir) {
		cli();
		INI_openCreate(&err);
		sei();
		if (err) return;
		for (uint8_t drv = 0; drv < 4; drv++) {
			err = 0;

			INI_read(buffer2.ini.ini, &err);
			if (err) { return; }
			FILE_openAbs(&buffer2.sd6031.d88[drv], (char *)buffer2.ini.ini+drv*64, 0, &err);
			if (err) { buffer2.sd6031.img[drv].valid = 0; buffer2.sd6031.d88[drv].valid = 0; continue; }
			if (memcmp(&buffer2.sd6031.d88[drv].name[8], "P31", 3) == 0) {
				buffer2.sd6031.img[drv]=buffer2.sd6031.d88[drv];
				buffer2.sd6031.d88[drv].valid = 0;
			} else {
				FILE_substituteFullpathExtwith((char *)buffer2.ini.ini+drv*64, "P31");
				FILE_openAbs(&buffer2.sd6031.img[drv], (char *)buffer2.ini.ini+drv*64, 0, &err);
				if (!err) { continue; }
				err = 0;
				if (!WP) {
#ifdef AIRDISP
				LCD_printAll("CreatingP31 file");
#endif
					// Notice : buffer2.sd.buf[512] is also used!
					cli();
					FILE_createAbs(&buffer2.sd6031.img[drv], (char *)buffer2.ini.ini+drv*64, 655360, &err);
					sei();
					if (err) { buffer2.sd6031.img[drv].valid=0; continue; }
					FILE_openAbs(&buffer2.sd6031.img[drv], (char *)buffer2.ini.ini+drv*64, 0, &err);
					if (err) { buffer2.sd6031.img[drv].valid=0; continue; }
					buffer2.sd6031.img[drv].protect = buffer2.sd6031.d88[drv].protect;
					d882p31(&buffer2.sd6031.img[drv], &buffer2.sd6031.d88[drv], &err);
					if (err) { buffer2.sd6031.img[drv].valid=0; continue; }
				} else {
#ifdef AIRDISP
					LCD_printAll("Write   protect.");
#endif
				}
			}
		}
	} else {
		for (uint8_t drive = 0; drive < 4; drive++) {
			uint8_t err = 0;
			
			struct FILE directory;
			char ext_[4]="   ", name_[9]="D       ";
			name_[1] = ('1'+drive);
			FILE_open(&directory, 0, name_, ext_, 1, &err);
			if (err) continue;

			FILE_open(&buffer2.sd6031.img[drive], directory.startCluster, "INIT    ", "P31", 1, &err);
			if (err) {
				err = 0;
				FILE_open(&buffer2.sd6031.img[drive], directory.startCluster, 0, "P31", 1, &err);
			}
			if (err) continue;

			//pc6031.error = 0;
		}
	}
}
